@@ -1148,7 +1148,7 @@ static int brcmf_usb_bus_setup(struct brcmf_usbdev_info *devinfo)
 	if (ret)
 		goto fail;
 
-	ret = brcmf_bus_start(devinfo->dev);
+	ret = brcmf_bus_started(devinfo->dev);
 	if (ret)
 		goto fail;
 
@@ -1158,18 +1158,17 @@ fail:
 	return ret;
 }
 
-static void brcmf_usb_probe_phase2(struct device *dev, int ret,
+static void brcmf_usb_probe_phase2(struct device *dev,
 				   const struct firmware *fw,
 				   void *nvram, u32 nvlen)
 {
 	struct brcmf_bus *bus = dev_get_drvdata(dev);
-	struct brcmf_usbdev_info *devinfo = bus->bus_priv.usb->devinfo;
-
-	if (ret)
-		goto error;
+	struct brcmf_usbdev_info *devinfo;
+	int ret;
 
 	brcmf_dbg(USB, "Start fw downloading\n");
 
+	devinfo = bus->bus_priv.usb->devinfo;
 	ret = check_file(fw->data);
 	if (ret < 0) {
 		brcmf_err("invalid firmware\n");
