@@ -73,14 +73,14 @@
 /* Highest firmware API version supported */
 #define IWL7260_UCODE_API_MAX	17
 #define IWL7265_UCODE_API_MAX	17
-#define IWL7265D_UCODE_API_MAX	26
-#define IWL3168_UCODE_API_MAX	26
+#define IWL7265D_UCODE_API_MAX	28
+#define IWL3168_UCODE_API_MAX	28
 
 /* Lowest firmware API version supported */
 #define IWL7260_UCODE_API_MIN	17
 #define IWL7265_UCODE_API_MIN	17
-#define IWL7265D_UCODE_API_MIN	22
-#define IWL3168_UCODE_API_MIN	22
+#define IWL7265D_UCODE_API_MIN	17
+#define IWL3168_UCODE_API_MIN	20
 
 /* NVM versions */
 #define IWL7260_NVM_VERSION		0x0a1d
@@ -108,7 +108,7 @@
 #define IWL3160_FW_PRE "/*(DEBLOBBED)*/"
 #define IWL3160_MODULE_FIRMWARE(api) IWL3160_FW_PRE /*(DEBLOBBED)*/
 
-#define IWL3168_FW_PRE "iwlwifi-3168-"
+#define IWL3168_FW_PRE "/*(DEBLOBBED)*/"
 #define IWL3168_MODULE_FIRMWARE(api) IWL3168_FW_PRE /*(DEBLOBBED)*/
 
 #define IWL7265_FW_PRE "/*(DEBLOBBED)*/"
